/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.ajax.renders;

import java.util.Map;

import org.json.JSONObject;
import org.mentawai.ajax.AjaxRender;
import org.mentawai.core.Action;
import org.mentawai.core.ConsequenceException;

/**
 * Expect a java.util.Map or a <a href="http://www.json.org/java/">org.json.JSONObject</a>
 * in the output.
 * 
 * @author Davi Luan Carneiro
 */
public class JSONMapAjaxRender implements AjaxRender {

	/** Default key */
	public static final String JSON_MAP_ATTR = "json_map";
	
	private String key = JSON_MAP_ATTR;
	
	public JSONMapAjaxRender() {}
	
	public JSONMapAjaxRender(String key) {
		this.key = key;
	}
	
	/**  
	 * @see org.mentawai.ajax.AjaxRender#renderize(org.mentawai.core.Action)
	 */
	public String renderize(Action action) throws Exception {
		Object obj = action.getOutput().getValue(key);
		if (obj == null) {
			throw new ConsequenceException("Invalid key: " + key);
		}
		
		JSONObject json;
		if (obj instanceof Map) {
			json = new JSONObject((Map)obj);
		} else if (obj instanceof JSONObject) {
			json = (JSONObject)obj;
		} else {
			throw new ConsequenceException("The value of key " + key + " isn't a Map or JSONObject");
		}
		
		return json.toString();
	}

    public String getContentType() {
        return TEXT_HTML;
    }
}
