package org.mentawai.ajax.renders;

import org.mentawai.ajax.AjaxRender;
import org.mentawai.core.Action;
import org.mentawai.core.Output;
/**
 * 
 * <p>A AjaxRender that creates a XML structure from
 * a given String.</p>
 * 
 * <p>By default, it gets from the action's output a String identified by the key "value" and
 * generates a XML structure with a sigle tag named "value", with the body content as the value
 * from the output, but this behavior can be changed at the constructor.</p>
 *   
 * <p>
 * Example of a generated XML:
 * <br/>
 * &ltvalue&gtvalueFromOutput&lt/value&gt
 * </p>
 * @author Rubem Azenha (rubem.azenha@gmail.com)
 *
 */
public class StringAjaxRender implements AjaxRender {
    private static final String DEFAULT_VALUE_ATTR = "value";
    private static final String DEFAULT_TAG_NAME= "value";

    private String valueAttr;
	private String tagName;
	
	public StringAjaxRender() {
		this.valueAttr = DEFAULT_VALUE_ATTR;
		this.tagName = DEFAULT_TAG_NAME;
	}
	
	public StringAjaxRender(String name, String attr) {
		tagName = name;
		valueAttr = attr;
	}
	
	public String renderize(Action action) throws Exception {
		Output output = action.getOutput();
		String value = (String) output.getValue(valueAttr);
		StringBuffer buffer = new StringBuffer();
		buffer.append('<').append(tagName).append('>').append(value)
		.append("</").append(tagName).append('>');
		String result = buffer.toString();
		return result;
	}

    public String getContentType() {
        return TEXT_XML;
    }


}
