/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.authorization;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.mentawai.action.BaseLoginAction;
import org.mentawai.core.Context;

/**
 * @author Sergio Oliveira
 */
public class AuthorizationManager implements Serializable {
    
    protected static Map<String, Group> groups = new HashMap<String, Group>();
    
    public static void addGroup(Group g) {
        groups.put(g.getName(), g);
    }
    
    public static boolean hasGroup(Group g) {
        return hasGroup(g.getName());
    }
    
    public static boolean hasGroup(String name) {
        return groups.containsKey(name);
    }
    
    public static boolean check(String group, String permission) {
        Group g = groups.get(group);
        if (g != null) {
            return g.hasPermission(permission);
        }
        return false;
    }
    
    public static boolean checkAny(String group, List<String> permissions) {
        Iterator<String> iter = permissions.iterator();
        while(iter.hasNext()) {
            
            String permission = iter.next();
            
            if (check(group, permission)) return true;
        }
        
        return false;
    }
    
    public static boolean checkAll(String group, List permissions) {
        
        Iterator iter = permissions.iterator();
        while(iter.hasNext()) {
            
            String permission = (String) iter.next();
            
            if (!check(group, permission)) return false;
        }
        
        return true;
    }
    
    public static void setUserGroups(List groups, Context session) {
        
        BaseLoginAction.setUserGroups(groups, session);
        
    }
    
    public static void setUserGroups(String groups, Context session) {
        
        BaseLoginAction.setUserGroups(groups, session);
    }
    
}