/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.filter;

import java.util.List;

import org.mentawai.action.BaseLoginAction;
import org.mentawai.authorization.Authorizable;
import org.mentawai.core.Action;
import org.mentawai.core.Filter;
import org.mentawai.core.InvocationChain;

/**
 * A filter to handle user authorization dynamically.
 * 
 * You should use this filter to protect your actions from unauthorized access.
 * 
 * @author Sergio Oliveira
 * @since 1.8
 */
public class DynAuthorizationFilter implements Filter {
	
	public static final String ACCESSDENIED = AuthorizationFilter.ACCESSDENIED;
    
    /**
     * Creates a new dynamic authorization filter.
     */
	public DynAuthorizationFilter() {

	}
	
	/**
	 * The default implementation of this method returns true for everything.
	 * 
	 * You can override this method to create the authorization logic for your entire application. 
	 * 
	 * @param action
	 * @param actionName
	 * @param innerAction
	 * @param user The user in the session (can be null)
	 * @param userGroups The user groups (can be null)
	 * @return true if authorized, false otherwise
	 */
	public boolean isAuthorized(Action action, String actionName, String innerAction, Object user, List userGroups) {
		
		return true;
		
	}
    
	public String filter(InvocationChain chain) throws Exception {
		
		Action action = chain.getAction();
		
		String actionName = chain.getActionName();
		
		String innerAction = chain.getInnerAction();
		
		Object user = BaseLoginAction.getUserSession(action.getSession());
		
		List groups = BaseLoginAction.getUserGroups(action.getSession());
		
		if (action instanceof Authorizable) {
			
			Authorizable a = (Authorizable) action;
			
			if (a.authorize(innerAction, user, groups)) {
				
				return chain.invoke();
				
			} else {
				
				return ACCESSDENIED;
			}
			
		} else {
			
			if (isAuthorized(action, actionName, innerAction, user, groups)) {
				
				return chain.invoke();
				
			} else {
				
				return ACCESSDENIED;
			}
		}
	}
	
    public void destroy() { }
	
}
		