/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.message;

import java.util.Iterator;
import java.util.Locale;
import java.util.Map;

/**
 * @author Sergio Oliveira
 */
public abstract class AbstractMessage implements Message {

	private static final String TOKEN_MARKER = "%";
	
    protected String id;
    protected MessageContext context;
	protected Map tokens = null;
	protected String tokenMarker = TOKEN_MARKER;
    
    public AbstractMessage(String id, MessageContext context) {
        this.id = id;
        this.context = context;
    }
	
	public AbstractMessage(String id, MessageContext context, Map tokens) {
		this(id, context);
		this.tokens = tokens;
	}
    
    public AbstractMessage(int id, MessageContext context) {
        this.id = String.valueOf(id);
        this.context = context;
    }
	
	public AbstractMessage(int id, MessageContext context, Map tokens) {
		this(id, context);
		this.tokens = tokens;
	}
    
    public String getId() { return id; }
    public MessageContext getContext() { return context; }
	
	public void setTokenMarker(String tokenMarker) {
		this.tokenMarker = tokenMarker;
	}
    
    public void  setTokens(Map tokens){
		this.tokens = tokens;
    }
    
    public String getText(Locale loc) {
		String s = context.getMessage(id, loc);
		if (tokens != null && !tokens.isEmpty()) {
			StringBuffer sb = new StringBuffer(s);
			Iterator iter = tokens.keySet().iterator();
			while(iter.hasNext()) {
				String tokenKey = (String) iter.next();
				String token = (String) tokens.get(tokenKey);
				StringBuffer t = new StringBuffer(tokenKey.length() + 2);
				t.append(tokenMarker).append(tokenKey).append(tokenMarker);
				String marker = t.toString();
				int index = sb.indexOf(marker);
				if(index >= 0) {
					sb.replace(index, index + marker.length(), token);
				} else {
					continue;
				}
			}
			return sb.toString();
		}
		return s;
    }
    
    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("Message: ID[").append(id).append("] CONTEXT[").append(context).append("]");
        return sb.toString();
    }
}

        
    