/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.tag;

import java.util.Locale;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.PageContext;
import javax.servlet.jsp.tagext.Tag;

import org.mentawai.formatter.Formatter;
import org.mentawai.formatter.FormatterManager;
import org.mentawai.list.ListData;
import org.mentawai.list.ListManager;
import org.mentawai.tag.util.Context;
import org.mentawai.tag.util.PrintTag;

/**
 * @author Sergio Oliveira
 */
public class Out extends PrintTag {

	private String value = null;
    private String list = null;
    private String formatter = null;

	public void setValue(String value) {
		this.value = value;
	}
    
    public void setList(String list) {
        this.list = list;
    }
    
    public void setFormatter(String formatter) {
    	this.formatter = formatter;
    }

    /*
	private static Object findValue(String value, PageContext pageContext) {
		if (action == null) return null;
		Output output = action.getOutput();
        Object obj = output.getValue(value);
        if (obj == null) {
            org.mentawai.core.Context session = action.getSession();
            obj = session.getAttribute(value);
            if (obj == null) {
                org.mentawai.core.Context application = action.getApplication();
                obj = application.getAttribute(value);
            }
        }
        return obj;
	}
    */
    
    public static Object getValue(Tag context, String value, PageContext pageContext, boolean tryBoolean) throws JspException {
        if (context != null) {
            Context ctx = (Context) context;
            Object obj = ctx.getObject();
            if (obj == null) {
                if (value == null) {
                    throw new JspException("Context is null!");
                } else {
                    Object object = getValue(value, pageContext, tryBoolean);
                    if (object == null) {
                        throw new JspException("Context is null!");
                    } else {
                        return object;
                    }
                }
            } else {
                if (value == null) {
                    // for loop...
                    return obj;
                } else {
                    Object object = null;
                    
                    /*
                    if (tryBoolean) {
                        object = getBooleanValue(obj, value);
                        if (object == null) {
                            object = getValue(obj, value, false);
                        }
                    } else {
                        object = getValue(obj, value, false);
                    }
                    */
                    
                    object = getValue(value, obj, tryBoolean);
                    
                    if (object != null) {
                        // for bean...
                        return object;
                    } else {
                        // try output...
                        object = getValue(value, pageContext, tryBoolean);
                        if (object == null) {
                            // not found in bean...
                            return "";
                        } else {
                            // for output...
                            return object;
                        }
                    }
                }
            }
        } else {
            if (value == null) {
                throw new JspException("Not inclosed by a context tag!");
            } else {
                //if (action == null) throw new JspException("No action in request for tag: " + value);
                Object obj = getValue(value, pageContext, tryBoolean);
                if (obj == null) return "";
                return obj;            
            }
        }   
    }
    
    public String getFromList(int i, Locale loc, String listname) throws JspException {
    	
    	return getFromList(i > 0 ? String.valueOf(i) : null, loc, listname);
    }
    
    public String getFromList(String i, Locale loc, String listname) throws JspException {
        ListData list = ListManager.getList(listname);
        if (list == null) throw new JspException("No data list for listname " + listname);
        String value = list.getValue(i, loc);
        if (value == null && i != null) {
            throw new JspException("No value inside listname " + listname + " for " + i + " / " + loc);
        }
        return value;
    }

	public String getStringToPrint() throws JspException {
		Tag parent = findAncestorWithClass(this, Context.class);
        Object obj = getValue(parent, value, pageContext, false);
        if (obj == null) throw new JspException("Out: No value can be found for " + value);
        
        if (list != null) {
            if (obj instanceof Integer) {
                Integer i = (Integer) obj;
                return getFromList(i.intValue(), loc, list);
            } else if (obj instanceof String) {
                try {
                    int i = Integer.parseInt(obj.toString());
                    return getFromList(i, loc, list);
                } catch(NumberFormatException e) {
                    throw new JspException("Value cannot be converted to int for list: " + list + " / " + obj.toString());
                }
            } else if (obj instanceof int[]) {
                int [] array = (int []) obj;
                StringBuffer sb = new StringBuffer(array.length * 40);
                for(int i=0;i<array.length;i++) {
                    String item = getFromList(array[i], loc, list);
                    if (i != 0) sb.append(", ");
                    sb.append(item);
                }
                return sb.toString();
            } else {
                throw new JspException("Value must be an int or an int array for list: " + value + " / " + obj);
            }
        }
        
        // formatter logic here:
        
        if (formatter != null) {
            
            Formatter f = FormatterManager.getFormatter(formatter);
            
            if (f == null) throw new JspException("Cannot find formatter: " + formatter);
            
            return f.format(obj, loc);
        	
        } else if (obj instanceof java.util.Date) {
        	
        	// if formatter was not defined but we have a default data formatter, then use it...
        	
        	Formatter f;
        	
        	if (( f = FormatterManager.getFixedDateFormatter()) != null) {
        		
        		return f.format(obj, loc);
        		
        	}
        	
        }
        
        return obj.toString();
	}
}