/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.tag.html;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.servlet.jsp.JspException;

import org.mentawai.list.ListData;
import org.mentawai.list.ListItem;
import org.mentawai.list.ListManager;
import org.mentawai.tag.Out;

/**
 * @author Sergio Oliveira
 */
public class Select extends HTMLTag {
	
	private String name;
	private String listname;
	private boolean noSelect = false;
	private int size = -1;
	private boolean emptyField = false;
	private String klass = null;
    private String style = null;
	private String id = null;
    private boolean multiple = false;
	
	public void setName(String name) { this.name = name; }
	public void setList(String listname) { this.listname = listname; }
	public void setNoSelect(boolean noSelect) { this.noSelect = noSelect; }
	public void setSize(int size) { this.size = size; }
	public void setEmptyField(boolean emptyField) { this.emptyField = emptyField; }
	public void setKlass(String klass) { this.klass = klass; }
    public void setStyle(String style) { this.style = style; }
	public void setId(String id) { this.id = id; }
    public void setMultiple(boolean multiple) { this.multiple = multiple; }
    
	protected StringBuffer buildTag() {
		StringBuffer sb = new StringBuffer(150);
		sb.append("<select name=\"");
		sb.append(name);
		sb.append("\"");
		if (size > 1) sb.append(" size=\"").append(size).append("\"");
		if (klass != null) sb.append(" class=\"").append(klass).append("\"");
        if (style != null) sb.append(" style=\"").append(style).append("\"");
		if (id != null) sb.append(" id=\"").append(id).append("\"");
        if (multiple) sb.append(" multiple=\"true\"");
        
        sb.append(getExtraAttributes());
        
		return sb;
	}
    
    public String getStringToPrint() throws JspException {
        
        String[] values = findValues(name);
        
        ListData list = null;
        
        Object obj = Out.getValue(listname, pageContext, false);
        
        if (obj != null && obj instanceof ListData) {
            
            list = (ListData) obj;
            
        } else if (obj != null && obj instanceof Map) {
        	
        	list = ListManager.convert(listname, (Map) obj);
        	
        } else if (obj != null && obj instanceof Collection) {
        	
        	list = ListManager.convert(listname, (Collection) obj);
            
        } else {
		
            list = ListManager.getList(listname);
            
        }
        
        if (list == null) throw new JspException("Cannot find list: " + listname);
        
        StringBuffer sb = new StringBuffer(list.size() * 50);
        
        List<ListItem> items = list.getValues(loc);
        
        if (items == null) {
        	
        	throw new JspException("Cannot find list for the locale: " + listname + " / " + loc);
        }
        
        Iterator<ListItem> iter = items.iterator();
        
        if (!noSelect) {
            sb.append(buildTag().toString());
            sb.append(">\n");
        }
		
		if (emptyField) {
			sb.append("<option value=\"0\"> - </option>\n");
		}
        
        while(iter.hasNext()) {
			ListItem item = iter.next();
			String id = item.getKey();
			String n = item.getValue();
			
            sb.append("<option value=\"");
            sb.append(id);
            sb.append("\"");
            if (contains(values, id)) {
                //sb.append(" SELECTED>");
                sb.append(" selected=\"true\">");
            } else {
                sb.append(">");
            }
            
            sb.append(n).append("</option>\n");
        }
        
        if (!noSelect) {
            sb.append("</select>\n");
        }
        
        return sb.toString();
    }
	
}
	