/*
 * Mentawai Web Framework http://mentawai.lohis.com.br/
 * Copyright (C) 2005  Sergio Oliveira Jr. (sergio.oliveira.jr@gmail.com)
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package org.mentawai.tag.html.dyntag.inputMoney;

import java.math.BigDecimal;
import java.text.DecimalFormat;

import javax.servlet.jsp.JspException;

import org.mentawai.tag.html.dyntag.inputText.InputText;

public class InputMoney extends InputText {

	private static final long serialVersionUID = 1L;

	private int decimals = 2;

	private String dec_point = ",";

	private String thousands_sep = ".";

	/** Method to build tag */
	public StringBuffer buildTag() {
		StringBuffer results = new StringBuffer(super.buildTag().toString());
		results.append(getTagClose());
		results.append("\n");
		results.append("\t\t<script>");
		results.append("inputMoneyMask(document.getElementById(\"" + getId() + "\"),"
				+ getDecimals() + ",\"" + getThousands_sep() + "\",\""
				+ getDec_point() + "\")");
		results.append("</script>");
		return results;
	}

	/*
	 * Formatador
	 */
	protected void prepareValue(StringBuffer results) {
		Object value = findObject(getName());
		if (value != null)
			results.append(" value=\"").append(
					customizeMask(getDecimals(), getThousands_sep(),
							getDec_point(), value)).append("\"");
		else if (getValue() != null)
			results.append(" value=\"").append(
					customizeMask(getDecimals(), getThousands_sep(),
							getDec_point(), getValue())).append("\"");
	}

	protected String makeZeros(int dec) {
		String zeroMask = "";
		for (int i = 0; i < dec; i++) {
			zeroMask += "0";
		}
		return zeroMask;
	}

	protected String formatNumber(Object object, int decimals) {
		if (object instanceof Float) {
			return new DecimalFormat("#,###." + makeZeros(decimals)).format((Float) object);
		} else if (object instanceof String) {
			return new DecimalFormat("#,###." + makeZeros(decimals)).format(new Float(object.toString()));
		}else if (object instanceof BigDecimal) {
 			return new DecimalFormat("#,##0." + makeZeros(decimals)).format((BigDecimal) object);
 		}
		return object.toString();
	}

	protected String customizeMask(int decimals, String thousands,String decimal, Object number) {
		if(number.toString().length() <= 0){
			return number.toString();
		}
		number = convert(number);
		String numero = formatNumber(number, decimals);
		String numeroFinal = "";
		int decimalSeparator = numero.indexOf(",");
		String numeroMilhares = numero.substring(0, decimalSeparator);
		String numeroDecimal = numero.substring(decimalSeparator + 1);
		numeroFinal = numeroMilhares.replace('.', thousands.charAt(0));
		numeroFinal = numeroFinal + decimal + numeroDecimal;
		return numeroFinal;
	}
	
	/*
	 * tenta arrumar o numero que foi digitado
	 */
	public Object convert(Object numero){
		if (numero instanceof String){
			Float numeroFloat;
			try {
				numeroFloat = new Float(numero.toString());
			} catch (NumberFormatException e){
				String numeroTo = numero.toString();
				if (numeroTo.indexOf(",") > 0){
					numeroTo = numeroTo.replace(".", "");
					numeroTo = numeroTo.replace(",", ".");
					numeroFloat = new Float(numeroTo);
					return numeroFloat;
				}
			} 
		}
		return numero;
	}

	/*
	 * fim formatador
	 */

	public String getStringToPrint() throws JspException {
		StringBuffer results = new StringBuffer(200);
		results.append(buildTag().toString());
		return results.toString();
	}

	/**
	 * @return Returns the dec_point.
	 */
	public String getDec_point() {
		return dec_point;
	}

	/**
	 * @param dec_point
	 *            The dec_point to set.
	 */
	public void setDec_point(String dec_point) {
		this.dec_point = dec_point;
	}

	/**
	 * @return Returns the decimals.
	 */
	public int getDecimals() {
		return decimals;
	}

	/**
	 * @param decimals
	 *            The decimals to set.
	 */
	public void setDecimals(int decimals) {
		this.decimals = decimals;
	}

	/**
	 * @return Returns the thousands_sep.
	 */
	public String getThousands_sep() {
		return thousands_sep;
	}

	/**
	 * @param thousands_sep
	 *            The thousands_sep to set.
	 */
	public void setThousands_sep(String thousands_sep) {
		this.thousands_sep = thousands_sep;
	}

}